#ifndef _KEYBOARD_HPP_
#define _KEYBOARD_HPP_
#include "sys.h"
/**
* Driver du clavier ;)
*/


class Keyboard {
	private:
		static Keyboard& instance;
		static char const * fr_keys_min; // Les touches minuscules francaises (scancode <=> code ascii)
		static char const * fr_keys_maj; // Les majuscules ...
		static char const * en_keys_min; // Idem en anglais
		static char const * en_keys_maj;

		char const * keysMin; // La config courante (pointe sur en_keys_min ou fr_keys_min)
		char const * keysMaj;

		bool capsEnabled;
		bool numEnabled;
		bool scrollEnabled;
		bool majPressed;
		bool controlPressed;
	public:

		unsigned int lastScanCode; // Derniere touche appuyée.

		/**
		* Default constructor
		*/
		Keyboard() // inlined
		{
			keysMin = fr_keys_min;
			keysMaj = fr_keys_maj;
			capsEnabled = false;
			numEnabled = true;
			scrollEnabled = false;
		}

		/**
		* Wait while keyboard is busy
		*/
		void wait() const // inlined too
		{ 
			for (int i = 0; i < 0x10000; i++) 
				if ((inb(0x64) & 0x02) == 0)
					break;
		}
		
		/**
		* Attendre le relachement de la touche pressée.
		*/
		void waitReleasingKey(int code) const
		{
			while ((code + 0x80) != inb(0x60))
				outb(0x20, 0x20);
		}

		/**
		* Wait for user entry.
		*/
		char getChar();

};

static inline void waitForever() {
	for(;;);
}

#endif /* _KEYBOARD_HPP_ */
