#include "../headers/types.h"
#include "../headers/sys.h"

#define GEN0_COUNTER 0x40
#define CLOCK_TICK_RATE 1193180
#define HZ 10
#define TIME (CLOCK_TICK_RATE / HZ)
/* we reprogram the 8253 timer chip (IRQ 0) to run at this frequency */
#define	HZ		100

extern void * adresseVideo;
extern char* horlogeScreen;
void initTimer();
void testCPP(void * s);
#define ICW1_M 	0x020 /* Master PIC (8259)*/
#define ICW2_M  0x021
#define ICW3_M  0x021
#define ICW4_M  0x021
#define OCW1_M  0x021
#define OCW2_M  0x020
#define OCW3_M  0x020

#define ICW1_S  0x0A0 /* Slave PIC*/
#define ICW2_S  0x0A1
#define ICW3_S  0x0A1
#define ICW4_S  0x0A1
#define OCW1_S  0x0A1
#define OCW2_S  0x0A0
#define OCW3_S  0x0A0

#define PIC1_BASE 0x020 /* Interrupt base for each PIC in HARTIK 3.0 */
#define PIC2_BASE 0x028
#define EOI       0x020 /* End Of Interrupt code for PIC! */

#define bit_on(v,b)     ((v) |= (1 << (b)))
#define bit_off(v,b)    ((v) &= ~(1 << (b)))

/* PIC interrupt mask */
u8 ll_PIC_master_mask = 0xFE;
u8 ll_PIC_slave_mask = 0xFE;


void PIC_init(void)
{
    outbp(ICW1_M,0x11);
    outbp(ICW2_M,PIC1_BASE);
    outbp(ICW3_M,0x04);
    outbp(ICW4_M,0x01);
    outbp(OCW1_M,0xFF);

    outbp(ICW1_S,0x11);
    outbp(ICW2_S,PIC2_BASE);
    outbp(ICW3_S,0x02);
    outbp(ICW4_S,0x01);
    outbp(OCW1_S,0xFF);
	
	/* affiche l'horloge */
	horlogeScreen[104] = '0'; 
	horlogeScreen[106] = '0'; 
	horlogeScreen[108] = 'h';  
	horlogeScreen[110] = '0';  
	horlogeScreen[112] = '0'; 
	horlogeScreen[114] = 'm';
	horlogeScreen[116] = '0';
	horlogeScreen[118] = '0';
	
	horlogeScreen[105] = 0xF;
	horlogeScreen[107] = 0xF;
	horlogeScreen[109] = 0xF;
	horlogeScreen[111] = 0xF;
	horlogeScreen[113] = 0xF;
	horlogeScreen[115] = 0xF;
	horlogeScreen[117] = 0xF;
	horlogeScreen[119] = 0xF;


}

void PIC_end(void)
{
    outbp(ICW1_M,0x11);
    outbp(ICW2_M,0x08);
    outbp(ICW3_M,0x04);
    outbp(ICW4_M,0x01);
    outbp(OCW1_M,0xFF);

    outbp(ICW1_S,0x11);
    outbp(ICW2_S,0x70);
    outbp(ICW3_S,0x02);
    outbp(ICW4_S,0x01);
    outbp(OCW1_S,0xFF);
}

void irq_mask(unsigned short irqno)
{
    /* Le Timer ne peut pas etre masquÈ */
    if (irqno == 0) return;
    /* L'interruption est dans le PIC maitre ?*/
    if (irqno < 8) {
	bit_on(ll_PIC_master_mask,irqno);
	outbp(0x21,ll_PIC_master_mask);
    } else if (irqno < 16) {
	/* L'interruption est dans le PIC esclave ?*/
	bit_on(ll_PIC_slave_mask,irqno-8);
	outbp(0xA1,ll_PIC_slave_mask);
	/* Si toutes les interruptions du PIC esclave sont dÈsactivÈes   */
	/* On dÈsactive ce dernier du PIC maitre via IRQ 2*/
	if (ll_PIC_slave_mask == 0xFF && !(ll_PIC_master_mask & 0x04)) {
	    bit_on(ll_PIC_master_mask,2);
	    outbp(0x21,ll_PIC_master_mask);
	}
    }
}

void irq_unmask(unsigned short irqno)
{
    /* Le Timer ne peut pas etre demasquÈ */
    if (irqno == 0) return;
    /* PIC maitre */
    if (irqno < 8) {
	bit_off(ll_PIC_master_mask,irqno);
	outbp(0x21,ll_PIC_master_mask);
    } else if (irqno < 16) {
	/* PIC esclave */
	bit_off(ll_PIC_slave_mask,irqno-8);
	outbp(0xA1,ll_PIC_slave_mask);
	/* Activation du PIC esclave si necessaire */
	if (ll_PIC_master_mask & 0x04) {
	    bit_off(ll_PIC_master_mask,2);
	    outbp(0x21,ll_PIC_master_mask);
	}
    }
}

void initTimer()
{
	static const unsigned short period = (3579545L / 3) / HZ;
		/* reprogram the 8253 timer chip to run at 'HZ', instead of 18 Hz */
	outb(0x43, 0x36);	/* channel 0, LSB/MSB, mode 3, binary */
	outb(0x40, period & 0xFF);	/* LSB */
	outb(0x40, period >> 8);	/* MSB */
}

