#include "../headers/iostream.hpp"
#include "../headers/video.hpp"
#include "../headers/keyboard.hpp"
#include "../headers/system.hpp"

extern "C"{
void init_gdt(void);
void init_idt(void);
void init_pic(void);
void waitForKey();
void initTimer();
void PIC_init();
void irq_mask(unsigned short irqno);
void irq_unmask(unsigned short irqno);
}

extern unsigned int lastScanCodeKey;

Keyboard System::getKeyboard(){
	return keyboard;
}

void System::launchShell(int nb)
{
	__asm("cli");
	if(numShellActif != nb){		
		getShell(numShellActif).saveBuffer(vid);
		vid.clear();
		numShellActif = nb;
		switch(nb)
		{
			case 0:
				getShell(nb).draw(vid,getShell(nb).getColor(),"0");
			break;
			case 1:
				getShell(nb).draw(vid,getShell(nb).getColor(),"1");
			break;
			case 2:
				getShell(nb).draw(vid,getShell(nb).getColor(),"2");
			break;
		}
		
	}
	__asm("sti");
}

Shell& System::getShell(int num){
	return tabShell[num];
}

System::System(){
	init_gdt();
	vid << "Chargement GDT ............................................... " << vid.green << "[OK]" << std::endl;
	init_idt();
	vid << "Chargement IDT ............................................... " << vid.green << "[OK]" << std::endl;
	PIC_init();
	irq_unmask(1);
	vid << "Reprogrammation des PICs ..................................... " << vid.green << "[OK]" << std::endl;
	sti;
	vid << "Activation des Interruptions Materielles ..................... " << vid.green << "[OK]" << std::endl;
	initTimer();
	vid << "Initialisation pile .......................................... " << vid.green << "[OK]" << std::endl;
	vid << "Chargement Multitache ........................................ " << vid.red << "[failed]" << std::endl;
	vid << "Chargement Thread ............................................ " << vid.blue << "[?]" << std::endl;
	vid.clear();
	numShellActif = 0;
	getShell(numShellActif).draw(vid,vid.blue,"0");
	getShell(1).setColor(vid.green);
	getShell(2).setColor(vid.white);
	while(1)
	{
		char touche = getKeyboard().getChar();
		if(touche == 'b')
		{
			launchShell(2);
		}
		else
		if(touche == 'v')
		{
			launchShell(1);
		}
		else
		if(touche == 'x')
		{
			launchShell(0);
		}
		else
			getShell(numShellActif).setTouche(touche,vid);
	}
}


