#ifndef ECRAN_HPP
#define ECRAN_HPP

#define BG_BLACK	0x0
#define FG_BLACK	0x0
#define FG_BLUE 	0x1
#define FG_GREEN 	0x2
#define FG_RED 		0x4
#define FG_CLEAR 	0x8
#define BG_BLUE 	0x10
#define BG_GREEN 	0x20
#define BG_RED 		0x40
#define BLINK		0x80

#define FG_WHITE	(FG_BLUE | FG_GREEN | FG_RED)
#define FG_PURPLE	(FG_BLUE | FG_RED)
#define FG_YELLOW	(FG_BLUE | FG_GREEN)

#define BG_WHITE	(BG_BLUE | BG_GREEN | BG_RED)
#define BG_PURPLE	(BG_BLUE | BG_RED)
#define BG_YELLOW	(BG_BLUE | BG_GREEN)

static inline unsigned char blink(int c) {
	return c | BLINK;
}

static inline unsigned char unblink(int c) {
	return c & ~BLINK;
}

/**
 * Permet la gestion de l'affichage bas niveau sur le système d'exploitation.
 */
class Video
{
	private:

		/**
		 * Attribut d'un pixel(couleur_fond, couleur_car, clignotement)
		 */
		unsigned char defaultAttributes; 

		/**
		 * Adresse du début de la vidéo. 
		 * La carte graphique lit dans cette zone pour afficher.
		 */
		char * video;

		/**
		 * Position courante du curseur; 
		 * Varie entre 0 et (25 * 80 - 1)
		 */
		int emplacement;
		void Decale();
	public:

		unsigned char white;
		unsigned char red;
		unsigned char green;
		unsigned char blue;

		Video();
		void clear();
		void put(int n, unsigned char attributes = unblink(FG_WHITE | BG_BLACK));
		void put(char const * msg, unsigned char attributes = unblink(FG_WHITE | BG_BLACK));
		void put(char c, unsigned char attributes = unblink(FG_WHITE | BG_BLACK));
		void error(char const * msg)
		{
			put(msg, unblink(BG_BLACK | FG_RED));
		}

		void success(char const * msg)
		{
			put(msg, unblink(BG_BLACK | FG_GREEN));
		}

		Video& operator<<(unsigned char attribute)
		{
			//put("!!", unblink(BG_BLACK | FG_WHITE));
			defaultAttributes = attribute;
			return *this;
		}
		
		Video& operator<<(int n)
		{
			put(n, defaultAttributes);
			defaultAttributes = unblink(BG_BLACK | FG_WHITE);
			return *this;
		}

		Video& operator<<(char const * msg)
		{
			put(msg, defaultAttributes);
			defaultAttributes = unblink(BG_BLACK | FG_WHITE);
			return *this;
		}

		Video& operator<<(char c)
		{
			put(c, defaultAttributes);
			defaultAttributes = unblink(BG_BLACK | FG_WHITE);
			return *this;
		}

		void setCursor();
		int getEmplacement();
		void setEmplacement(int emp);
		char* getBufferVideo();
		void setPixel(int num,int value);
		char getPixel(int num);
};

#endif
